@@ -140,6 +140,14 @@ class App:
         """
         return self.get_component(self.app_id + '-info')
 
+    def is_available(self) -> bool:
+        """Return whether the app is available to install."""
+        for component in self.components.values():
+            if not component.is_available():
+                return False
+
+        return True
+
     def setup(self, old_version):
         """Install and configure the app and its components."""
         for component in self.components.values():
@@ -147,7 +155,9 @@ class App:
 
     def uninstall(self):
         """De-configure and uninstall the app."""
-        for component in self.components.values():
+        # Remove components in the reverse order so that dependencies among
+        # components is properly satisfied.
+        for component in reversed(self.components.values()):
             component.uninstall()
 
     def get_setup_state(self) -> SetupState:
@@ -328,6 +338,10 @@ class Component:
         """
         return App.get(self.app_id)
 
+    def is_available(self) -> bool:
+        """Return whether the app is available to install."""
+        return True
+
     def setup(self, old_version):
         """Run operations to install and configure the component."""
 
@@ -582,6 +596,9 @@ class EnableState(LeaderComponent):
 
 def apps_init():
     """Create apps by constructing them with components."""
+    if App.list():
+        return  # Apps have already been initialized
+
     from . import module_loader  # noqa  # Avoid circular import
     for module_name, module in module_loader.loaded_modules.items():
         _initialize_module(module_name, module)
