@@ -6,6 +6,7 @@ pytest configuration for all tests.
 import importlib
 import os
 import pathlib
+import subprocess
 from unittest.mock import patch
 
 import pytest
@@ -110,17 +111,6 @@ def fixture_needs_sudo():
         pytest.skip('Needs sudo command installed.')
 
 
-@pytest.fixture(name='no_privileged_server', scope='module')
-def fixture_no_privileged__server():
-    """Don't setup for and run privileged methods on server.
-
-    Tests on using privileged daemon are not yet implemented.
-    """
-    with patch('plinth.actions._run_privileged_method_on_server') as mock:
-        mock.side_effect = NotImplementedError
-        yield
-
-
 @pytest.fixture(scope='session')
 def splinter_selenium_implicit_wait():
     """Disable implicit waiting."""
@@ -163,9 +153,10 @@ def fixture_mock_privileged(request):
         privileged_modules_to_mock = request.module.privileged_modules_to_mock
     except AttributeError:
         raise AttributeError(
-            'mock_privileged fixture requires "privileged_module_to_mock" '
+            'mock_privileged fixture requires "privileged_modules_to_mock" '
             'attribute at module level')
 
+    mocked_methods = []
     for module_name in privileged_modules_to_mock:
         module = importlib.import_module(module_name)
         for name, member in module.__dict__.items():
@@ -176,19 +167,32 @@ def fixture_mock_privileged(request):
             if not getattr(member, '_privileged', False):
                 continue
 
-            setattr(wrapped, '_original_wrapper', member)
-            module.__dict__[name] = wrapped
+            while getattr(wrapped, '__wrapped__', None) and getattr(
+                    wrapped, '_privileged', True):
+                wrapped = getattr(wrapped, '__wrapped__', None)
+
+            setattr(wrapped, '_skip_privileged_call', True)
+            mocked_methods.append(wrapped)
 
     yield
 
-    for module_name in privileged_modules_to_mock:
-        module = importlib.import_module(module_name)
-        for name, member in module.__dict__.items():
-            wrapper = getattr(member, '_original_wrapper', None)
-            if not callable(member) or not wrapper:
-                continue
+    for mocked_method in mocked_methods:
+        try:
+            delattr(mocked_method, '_skip_privileged_call')
+        except AttributeError:
+            pass
 
-            module.__dict__[name] = wrapper
+
+@pytest.fixture(name='mock_run_as_user')
+def fixture_mock_run_as_user():
+    """A fixture to override action_utils.run_as_user."""
+
+    def _bypass_runuser(*args, username, **kwargs):
+        return subprocess.run(*args, **kwargs)
+
+    with patch('plinth.action_utils.run_as_user') as mock:
+        mock.side_effect = _bypass_runuser
+        yield
 
 
 @pytest.fixture(name='splinter_screenshot_dir', scope='session')
