@@ -14,29 +14,58 @@ from django.utils.translation import gettext_lazy as _
 import plinth
 
 
+def _get_repository_choices():
+    """Return the list of available repositories."""
+    import plinth.modules.backups.repository as repository_module
+    choices = [(repository.uuid, repository.name)
+               for repository in repository_module.get_repositories()
+               if repository.is_usable()]
+
+    return choices
+
+
 class AppEnableDisableForm(forms.Form):
     """Form to enable / disable an app."""
     should_enable = forms.BooleanField(widget=forms.HiddenInput,
                                        required=False)
 
 
-class DomainSelectionForm(forms.Form):
-    """Form for selecting a domain name to be used for
-    distributed federated applications
-    """
+class UninstallForm(forms.Form):
+    """Form to uninstall an app."""
+    should_backup = forms.BooleanField(
+        label=_('Backup app before uninstall'),
+        help_text=_('Restoring from the backup will restore app data.'),
+        required=False, initial=True)
+    repository = forms.ChoiceField(label=_('Repository to backup to'),
+                                   choices=_get_repository_choices)
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, **kwargs):
+        """Determine if backup fields must be used."""
+        has_backup_restore = kwargs.pop('has_backup_restore', True)
+        super().__init__(**kwargs)
+        if not has_backup_restore:
+            del self.fields['should_backup']
+            del self.fields['repository']
+
+
+class DomainSelectionForm(forms.Form):
+    """Form for selecting a domain name."""
+
+    def __init__(self, show_none=False, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
         from plinth.modules.names.components import DomainName
         domains = list(DomainName.list_names())
-        self.fields['domain_name'].choices = zip(domains, domains)
+
+        choices = list(zip(domains, domains))
+        if show_none:
+            choices = [('', _('(None)'))] + choices
+            self.fields['domain_name'].required = False
+
+        self.fields['domain_name'].choices = choices
 
     domain_name = forms.ChoiceField(
-        label=_('Select a domain name to be used with this application'),
-        help_text=_(
-            'Warning! The application may not work properly if domain name is '
-            'changed later.'), choices=[])
+        label=_('Select a domain name to be used with this application'))
 
 
 def _get_domain_choices():
