@@ -12,7 +12,7 @@ import tarfile
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import action_utils
+from plinth import action_utils, actions
 from plinth import app as app_module
 from plinth import module_loader
 from plinth.actions import privileged, secret_str
@@ -118,8 +118,8 @@ def reraise_known_errors(privileged_func):
 
 def _reraise_known_errors(err):
     """Look whether the caught error is known and reraise it accordingly"""
-    stdout = getattr(err, 'stdout', b'').decode()
-    stderr = getattr(err, 'stderr', b'').decode()
+    stdout = (getattr(err, 'stdout') or b'').decode()
+    stderr = (getattr(err, 'stderr') or b'').decode()
     caught_error = str((err, err.args, stdout, stderr))
     for known_error in KNOWN_ERRORS:
         for error in known_error['errors']:
@@ -146,12 +146,12 @@ def mount(mountpoint: str, remote_path: str, ssh_keyfile: str | None = None,
     # 'reconnect', 'ServerAliveInternal' and 'ServerAliveCountMax' allow the
     # client (FreedomBox) to keep control of the SSH connection even when the
     # SSH server misbehaves. Without these options, other commands such as
-    # '/usr/share/plinth/actions/actions storage usage_info --no-args', 'df',
-    # '/usr/share/plinth/actions/actions sshfs is_mounted --no-args', or
-    # 'mountpoint' might block indefinitely (even when manually invoked from
-    # the command line). This situation has some lateral effects, causing major
-    # system instability in the course of ~11 days, and leaving the system in
-    # such state that the only solution is a reboot.
+    # 'freedombox-cmd storage usage_info --no-args', 'df', 'freedombox-cmd
+    # sshfs is_mounted --no-args', or 'mountpoint' might block indefinitely
+    # (even when manually invoked from the command line). This situation has
+    # some lateral effects, causing major system instability in the course of
+    # ~11 days, and leaving the system in such state that the only solution is
+    # a reboot.
     cmd = [
         'sshfs', remote_path, mountpoint, '-o',
         f'UserKnownHostsFile={user_known_hosts_file}', '-o',
@@ -166,14 +166,14 @@ def mount(mountpoint: str, remote_path: str, ssh_keyfile: str | None = None,
         cmd += ['-o', 'password_stdin']
         input_ = password.encode()
 
-    subprocess.run(cmd, check=True, timeout=TIMEOUT, input=input_)
+    action_utils.run(cmd, check=True, timeout=TIMEOUT, input=input_)
 
 
 @reraise_known_errors
 @privileged
 def umount(mountpoint: str):
     """Unmount a mountpoint."""
-    subprocess.run(['umount', mountpoint], check=True)
+    action_utils.run(['umount', mountpoint], check=True)
 
 
 def _validate_mountpoint(mountpoint):
@@ -340,8 +340,11 @@ def _extract(archive_path, destination, encryption_passphrase, locations=None):
 @privileged
 def export_tar(path: str, encryption_passphrase: secret_str | None = None):
     """Export archive contents as tar stream on stdout."""
-    _run(['borg', 'export-tar', path, '-', '--tar-filter=gzip'],
-         encryption_passphrase)
+    env = _get_env(encryption_passphrase)
+    process = subprocess.Popen(
+        ['borg', 'export-tar', path, '-', '--tar-filter=gzip'], env=env,
+        stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+    return actions.ProcessBufferedReader(process)
 
 
 def _read_archive_file(archive, filepath, encryption_passphrase):
@@ -512,4 +515,4 @@ def _get_env(encryption_passphrase: str | None = None):
 def _run(cmd, encryption_passphrase=None, check=True, **kwargs):
     """Wrap the command with extra encryption passphrase handling."""
     env = _get_env(encryption_passphrase)
-    return subprocess.run(cmd, check=check, env=env, **kwargs)
+    return action_utils.run(cmd, check=check, env=env, **kwargs)
