@@ -3,7 +3,6 @@
 
 import abc
 import datetime
-import io
 import logging
 import os
 import subprocess
@@ -177,39 +176,10 @@ class BaseBorgRepository(abc.ABC):
 
     def get_download_stream(self, archive_name):
         """Return an stream of .tar.gz binary data for a backup archive."""
-
-        class BufferedReader(io.BufferedReader):
-            """Improve performance of buffered binary streaming.
-
-            Django simply returns the iterator as a response for the WSGI app.
-            CherryPy then iterates over this iterator and writes to HTTP
-            response. This calls __next__ over the BufferedReader that is
-            process.stdout. However, this seems to call readline() which looks
-            for \n in binary data which leads to short unpredictably sized
-            chunks which in turn lead to severe performance degradation. So,
-            overwrite this and call read() which is better geared for handling
-            binary data.
-
-            """
-
-            def __next__(self):
-                """Override to call read() instead of readline()."""
-                chunk = self.read(io.DEFAULT_BUFFER_SIZE)
-                if not chunk:
-                    raise StopIteration
-
-                return chunk
-
-        proc, read_fd, input_ = privileged.export_tar(
-            self._get_archive_path(archive_name),
-            self._get_encryption_passpharse(), _raw_output=True)
-
-        os.close(read_fd)  # Don't use the pipe for communication, just stdout
-        proc.stdin.write(input_)
-        proc.stdin.close()
-        proc.stderr.close()  # writing to stderr in child will cause SIGPIPE
-
-        return BufferedReader(proc.stdout)
+        reader = privileged.export_tar(self._get_archive_path(archive_name),
+                                       self._get_encryption_passpharse(),
+                                       _raw_output=True)
+        return reader
 
     def _get_archive_path(self, archive_name):
         """Return full borg path for an archive."""
@@ -223,6 +193,11 @@ class BaseBorgRepository(abc.ABC):
 
         return None
 
+    def generate_archive_name(self):
+        """Return a name to create a backup archive with using time."""
+        return datetime.datetime.now().astimezone().replace(
+            microsecond=0).isoformat()
+
     def get_archive_apps(self, archive_name):
         """Get list of apps included in an archive."""
         archive_path = self._get_archive_path(archive_name)
