@@ -59,15 +59,13 @@ class KiwixApp(app_module.App):
             donation_url='https://www.kiwix.org/en/support-us/')
         self.add(info)
 
-        menu_item = menu.Menu('menu-kiwix', info.name, info.short_description,
-                              info.icon_filename, 'kiwix:index',
-                              parent_url_name='apps')
+        menu_item = menu.Menu('menu-kiwix', info.name, info.icon_filename,
+                              info.tags, 'kiwix:index', parent_url_name='apps')
         self.add(menu_item)
 
         shortcut = frontpage.Shortcut('shortcut-kiwix', info.name,
-                                      short_description=info.short_description,
                                       icon=info.icon_filename, url='/kiwix',
-                                      clients=info.clients,
+                                      clients=info.clients, tags=info.tags,
                                       login_required=False,
                                       allowed_groups=list(groups))
         self.add(shortcut)
