@@ -8,7 +8,7 @@ from django.urls import re_path
 from . import views
 
 urlpatterns = [
-    re_path(r'^sys/networks/$', views.index, name='index'),
+    re_path(r'^sys/networks/$', views.NetworksAppView.as_view(), name='index'),
     re_path(r'^sys/networks/(?P<uuid>[\w.@+-]+)/show/$', views.show,
             name='show'),
     re_path(r'^sys/networks/(?P<uuid>[\w.@+-]+)/edit/$', views.edit,
@@ -32,18 +32,9 @@ urlpatterns = [
     re_path(r'^sys/networks/router-configuration/$',
             views.RouterConfigurationView.as_view(),
             name='router-configuration'),
-    re_path(r'^sys/networks/firstboot/router-configuration/$',
-            views.RouterConfigurationFirstBootView.as_view(),
-            name='router-configuration-first-boot'),
     re_path(r'^sys/networks/internet-connection-type/$',
             views.InternetConnectionTypeView.as_view(),
             name='internet-connection-type'),
-    re_path(r'^sys/networks/firstboot/internet-connection-type/$',
-            views.InternetConnectionTypeFirstBootView.as_view(),
-            name='internet-connection-type-first-boot'),
     re_path(r'^sys/networks/network-topology/$',
             views.NetworkTopologyView.as_view(), name='network-topology'),
-    re_path(r'^sys/networks/firstboot/network-topology-first-boot/$',
-            views.NetworkTopologyFirstBootView.as_view(),
-            name='network-topology-first-boot'),
 ]
