@@ -1,7 +1,5 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-Views for samba module.
-"""
+"""Views for samba module."""
 
 import logging
 import os
@@ -15,9 +13,10 @@ from django.utils.translation import gettext as _
 from django.views.decorators.http import require_POST
 
 from plinth import views
-from plinth.errors import ActionError
 from plinth.modules import samba, storage
 
+from . import privileged
+
 logger = logging.getLogger(__name__)
 
 
@@ -28,17 +27,19 @@ def get_share_mounts():
 
     for mount in storage.get_mounts():
         mount_point = mount['mount_point']
-        if mount_point not in ignore_mounts:
-            basename = os.path.basename(mount_point)
-            mount['name'] = basename or _('FreedomBox OS disk')
-            mount['share_name_prefix'] = basename or 'disk'
-            mounts.append(mount)
+        if mount_point in ignore_mounts or os.path.isfile(mount_point):
+            continue
+        basename = os.path.basename(mount_point)
+        mount['name'] = basename or _('FreedomBox OS disk')
+        mount['share_name_prefix'] = basename or 'disk'
+        mounts.append(mount)
 
     return sorted(mounts, key=lambda k: k['mount_point'])
 
 
 class SambaAppView(views.AppView):
     """Samba sharing basic configuration."""
+
     app_id = 'samba'
     template_name = 'samba.html'
 
@@ -48,7 +49,7 @@ class SambaAppView(views.AppView):
         disks = get_share_mounts()
         context['disks'] = disks
 
-        shares = samba.get_shares()
+        shares = privileged.get_shares()
         shared_mounts = defaultdict(list)
         for share in shares:
             shared_mounts[share['mount_point']].append(share['share_type'])
@@ -100,7 +101,7 @@ def share(request, mount_point):
             try:
                 samba.add_share(mount_point, share_type, filesystem)
                 messages.success(request, _('Share enabled.'))
-            except ActionError as exception:
+            except Exception as exception:
                 logger.exception('Error enabling share')
                 messages.error(
                     request,
@@ -108,9 +109,9 @@ def share(request, mount_point):
                         error_message=exception))
         elif action == 'disable':
             try:
-                samba.delete_share(mount_point, share_type)
+                privileged.delete_share(mount_point, share_type)
                 messages.success(request, _('Share disabled.'))
-            except ActionError as exception:
+            except Exception as exception:
                 logger.exception('Error disabling share')
                 messages.error(
                     request,
