@@ -31,7 +31,7 @@ def _is_container():
     return process.returncode == 0
 
 
-pytestmark = pytest.mark.usefixtures('mock_privileged')
+pytestmark = pytest.mark.usefixtures('mock_privileged', 'mock_run_as_user')
 privileged_modules_to_mock = ['plinth.modules.storage.privileged']
 skip_if_container = pytest.mark.skipif(_is_container(),
                                        reason='running inside a container')
@@ -337,9 +337,11 @@ def _assert_validate_directory(path, error, check_writable=False,
         match = None if not error.args else error.args[0]
         with pytest.raises(error.__class__, match=match):
             privileged.validate_directory(path, check_creatable,
-                                          check_writable)
+                                          check_writable,
+                                          for_user='test-non-existent')
     else:
-        privileged.validate_directory(path, check_creatable, check_writable)
+        privileged.validate_directory(path, check_creatable, check_writable,
+                                      for_user='test-non-existent')
 
 
 @pytest.mark.usefixtures('needs_not_root')
@@ -375,7 +377,16 @@ def test_validate_directory_creatable(path, error):
 @patch('psutil.disk_partitions')
 def test_is_partition_read_only(disk_partitions):
     """Test whether checking for ro partition works."""
-    partition = psutil._common.sdiskpart  # pylint: disable=protected-access
+
+    def partition(*args):
+        try:
+            # psutil <= 5.9.8
+            # pylint: disable=protected-access
+            return psutil._common.sdiskpart(*args)
+        except TypeError:
+            # psutil >= 7.0
+            return psutil._common.sdiskpart(*args[:-2])
+
     disk_partitions.return_value = [
         partition('/dev/root', '/', 'btrfs', 'rw,nosuid', 42, 42),
         partition('/dev/root', '/foo', 'btrfs', 'rw', 321, 321),
