@@ -20,7 +20,7 @@ from plinth.diagnostic_check import DiagnosticCheck, Result
 from plinth.modules.backups.components import BackupRestore
 from plinth.package import Packages
 
-from . import distupgrade, manifest, privileged
+from . import distupgrade, manifest, privileged, utils
 
 first_boot_steps = [
     {
@@ -55,7 +55,7 @@ class UpgradesApp(app_module.App):
 
     app_id = 'upgrades'
 
-    _version = 19
+    _version = 20
 
     can_be_disabled = False
 
@@ -212,6 +212,7 @@ def setup_repositories(_):
     """Setup apt repositories for backports."""
     if is_backports_requested():
         privileged.activate_backports(cfg.develop)
+        privileged.activate_unstable()
 
 
 def check_dist_upgrade(_):
@@ -350,21 +351,12 @@ def is_backports_enabled():
     return os.path.exists(privileged.BACKPORTS_SOURCES_LIST)
 
 
-def get_current_release():
-    """Return current release and codename as a tuple."""
-    output = subprocess.check_output(
-        ['lsb_release', '--release', '--codename',
-         '--short']).decode().strip()
-    lines = output.split('\n')
-    return lines[0], lines[1]
-
-
 def is_backports_current():
     """Return whether backports are enabled for the current release."""
     if not is_backports_enabled():
         return False
 
-    _, dist = get_current_release()
+    _, dist = utils.get_current_release()
     dist += '-backports'
     sources = sourceslist.SourcesList()
     for source in sources:
@@ -379,15 +371,12 @@ def can_activate_backports():
     if cfg.develop:
         return True
 
-    # Release will be 'n/a' in latest unstable and testing distributions.
-    release, _ = get_current_release()
-    return release not in ['unstable', 'testing', 'n/a']
+    return not utils.is_distribution_unstable()
 
 
 def can_enable_dist_upgrade():
     """Return whether dist upgrade can be enabled."""
-    release, _ = get_current_release()
-    return release not in ['unstable', 'testing', 'n/a']
+    return not utils.is_distribution_rolling()
 
 
 def _diagnose_held_packages():
