@@ -464,6 +464,10 @@ def install(browser, app_name):
             browser.visit(browser.url)
         elif browser.is_element_present_by_css('.alert-danger'):
             break
+        elif (browser.is_element_present_by_css('.app-checking-availability')
+              and not browser.find_by_css(
+                  '.app-checking-availability').has_class('d-none')):
+            time.sleep(0.1)
         elif browser.is_element_present_by_css(install_button_css):
             install_button = browser.find_by_css(install_button_css).first
             if install_button['disabled']:
@@ -592,16 +596,40 @@ def running_inside_container():
 #############################
 # System -> Names utilities #
 #############################
-def set_hostname(browser, hostname):
+def set_hostname(browser, hostname: str):
+    """Configure the system hostname."""
     visit(browser, '/plinth/sys/names/hostname/')
     browser.find_by_id('id_hostname-hostname').fill(hostname)
     submit(browser, form_class='form-hostname')
 
 
-def set_domain_name(browser, domain_name):
-    visit(browser, '/plinth/sys/names/domains/')
-    browser.find_by_id('id_domain-name-domain_name').fill(domain_name)
-    submit(browser, form_class='form-domain-name')
+def domain_add(browser, domain_name: str):
+    """Add a domain to list of domains."""
+    if domain_name in domain_list(browser):
+        return
+
+    visit(browser, '/plinth/sys/names/')
+    click_link_by_href(browser, '/plinth/sys/names/domains/')
+    browser.find_by_id('id_domain-add-domain_name').fill(domain_name)
+    submit(browser, form_class='form-domain-add')
+
+
+def domain_remove(browser, domain_name: str):
+    """Remove a domain from list of domains."""
+    if domain_name not in domain_list(browser):
+        return
+
+    visit(browser, '/plinth/sys/names/')
+    click_link_by_href(browser,
+                       f'/plinth/sys/names/domains/{domain_name}/delete/')
+    submit(browser, form_class='form-delete')
+
+
+def domain_list(browser) -> list[str]:
+    """Return a list of domains configured."""
+    visit(browser, '/plinth/sys/names/')
+    elements = browser.find_by_css('td.names-domain-column')
+    return [element.text for element in elements]
 
 
 ##############################
